/*
 * Copyright (C) 2008 Luke Kenneth Casson Leighton <lkcl@lkcl.net>
 * Copyright (C) 2008 Alp Toker <alp@atoker.com>
 * Copyright (C) 2008 Apple Inc.
 */

#include "config.h"

#include <wtf/GetPtr.h>
#include "CSSRule.h"
#include "CSSValue.h"
#include "CSSVariablesDeclaration.h"
#include "CanvasPixelArray.h"
#include "DOMWindow.h"
#include "Element.h"
#include "Event.h"
#include "GDOMBinding.h"
#include "GDOMObject.h"
#include "HTMLCollection.h"
#include "Node.h"
#include "StyleSheet.h"
#include "Text.h"
#include "gdom/GStringConvert.h"
#include "gdom/GdomCSSVariablesDeclaration.h"
#include "gdom/GdomDOMObject.h"
#include "gdom/PrivateGdomCSSVariablesDeclaration.h"
#include "gdom/PrivateGdomDOMObject.h"
#include "gdom/PrivateIntGdomCSSVariablesDeclaration.h"
#include "glib-object.h"

namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore

namespace WebCore {
gpointer toGDOM(WebCore::CSSVariablesDeclaration* obj)
{
    return WebCore::cacheDOMObject<WebCore::CSSVariablesDeclaration, GDOMCSSVariablesDeclaration>(obj);
}
} /* namespace WebCore */
extern "C" {

GDOMCSSVariablesDeclaration::GDOMCSSVariablesDeclaration(PassRefPtr<WebCore::CSSVariablesDeclaration> impl)
    : GDOMObject()
    , m_impl(impl)
{
}

GDOMCSSVariablesDeclaration::~GDOMCSSVariablesDeclaration()
{
    m_impl.get()->deref();
    WebCore::GDOMScriptInterpreter::forgetDOMObject(m_impl.get());

}

gchar *
gdom_css_variables_declaration_get_variable_value (GdomCSSVariablesDeclaration *thiz, gchar * variable_name)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::CSSVariablesDeclaration * item = (gdom_css_variables_declaration_get_private(thiz));
    g_return_val_if_fail (item, 0);
    g_return_val_if_fail (variable_name, 0);
    WebCore::String _g_variable_name = WebCore::String::fromUTF8(variable_name);
    gchar * res = _G_GET_UTF8(item->getVariableValue(_g_variable_name));
    return res;

}

gchar *
gdom_css_variables_declaration_remove_variable (GdomCSSVariablesDeclaration *thiz, gchar * variable_name)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::CSSVariablesDeclaration * item = (gdom_css_variables_declaration_get_private(thiz));
    g_return_val_if_fail (item, 0);
    g_return_val_if_fail (variable_name, 0);
    WebCore::String _g_variable_name = WebCore::String::fromUTF8(variable_name);
    WebCore::ExceptionCode ec = 0;
    gchar * res = _G_GET_UTF8(item->removeVariable(_g_variable_name, ec));
    return res;

}

void
gdom_css_variables_declaration_set_variable (GdomCSSVariablesDeclaration *thiz, gchar * variable_name, gchar * value)
{
    g_return_if_fail (thiz);
    WebCore::CSSVariablesDeclaration * item = (gdom_css_variables_declaration_get_private(thiz));
    g_return_if_fail (item);
    g_return_if_fail (variable_name);
    g_return_if_fail (value);
    WebCore::String _g_variable_name = WebCore::String::fromUTF8(variable_name);
    WebCore::String _g_value = WebCore::String::fromUTF8(value);
    WebCore::ExceptionCode ec = 0;
    item->setVariable(_g_variable_name, _g_value, ec);

}

gchar *
gdom_css_variables_declaration_item (GdomCSSVariablesDeclaration *thiz, gulong index)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::CSSVariablesDeclaration * item = (gdom_css_variables_declaration_get_private(thiz));
    g_return_val_if_fail (item, 0);
    gchar * res = _G_GET_UTF8(item->item(index));
    return res;

}

} /* extern "C" */


#include "config.h"
#include "glib-object.h"
#include <webkit/webkitdefines.h>
#include <webkit/webkitprivate.h> 
#include "DOMWindow.h" /* yukk: TODO - make this optional */
#include "Node.h" /* yukk: TODO - make this optional */
#include "NamedNodeMap.h" /* yukk: TODO - make this optional */
#include "CanvasPattern.h" /* yukk: TODO - make this optional */
#include "CanvasGradient.h" /* yukk: TODO - make this optional */
#include "CanvasStyle.h" /* yukk: TODO - make this optional */
#include "CanvasRenderingContext2D.h" /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomDOMObject.h> /* yukk: TODO - make this optional */
#include <gdom/GdomEventTargetNode.h> /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomEventTargetNode.h> /* yukk: TODO - make this optional on EventTarget*/
#include "CSSVariablesDeclaration.h"
#include <gdom/GdomCSSVariablesDeclaration.h>
#include <gdom/PrivateGdomCSSVariablesDeclaration.h>
#include <gdom/PrivateIntGdomCSSVariablesDeclaration.h>
#include <gdom/GdomCSSVariablesDeclarationprops.h>

#include "gdom/GStringConvert.h"

#include "CanvasPixelArray.h"
#include "Event.h"
#include "Text.h"
#include "CSSValue.h"
#include "CSSRule.h"
#include "HTMLCollection.h"
#include "StyleSheet.h"
#include "Document.h"
#include "webkitmarshal.h"
#include <gdom/GdomEvent.h>



namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore



G_BEGIN_DECLS

G_DEFINE_TYPE(GdomCSSVariablesDeclaration, gdom_css_variables_declaration, GDOM_TYPE_DOM_OBJECT);

/*
struct _GdomCSSVariablesDeclarationPrivate {
    class WebCore::CSSVariablesDeclaration *item;
};

#define GDOM_CSS_VARIABLES_DECLARATION_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), GDOM_TYPE_CSS_VARIABLES_DECLARATION, GdomCSSVariablesDeclarationPrivate))
*/
G_END_DECLS
WebCore::CSSVariablesDeclaration *
gdom_css_variables_declaration_get_private(GdomCSSVariablesDeclaration *request)
{
    GDOMCSSVariablesDeclaration *res;
    res = static_cast<GDOMCSSVariablesDeclaration*>(gdom_dom_object_get_private(GDOM_DOM_OBJECT(request)));
    return static_cast<WebCore::CSSVariablesDeclaration*>(res->impl());
}

G_BEGIN_DECLS

enum {
    BROWSER_EVENT,
    LAST_SIGNAL
};

static guint event_signals[LAST_SIGNAL] = { 0, };

enum {
    PROP_0,
    PROP_CSS_TEXT,
    PROP_LENGTH,
    PROP_PARENT_RULE,
};


static void gdom_css_variables_declaration_finalize(GObject* object)
{
    /*GdomCSSVariablesDeclaration* request = GDOM_CSS_VARIABLES_DECLARATION(object);*/
    /*GdomCSSVariablesDeclarationPrivate* priv = request->priv;*/

    /*GObjectClass *parent_class = g_type_class_peek (g_type_peek_parent (GDOM_TYPE_DOM_OBJECT));*/

    /* Free any memory or close any files */

    /*parent_class-> finalize (object);*/

    G_OBJECT_CLASS(gdom_css_variables_declaration_parent_class)->finalize(object);
}

static void gdom_css_variables_declaration__set_property_fn(GObject * object, guint prop_id,
                     const GValue * value,
                     GParamSpec * pspec)
{
    GdomCSSVariablesDeclaration *self = GDOM_CSS_VARIABLES_DECLARATION(object);
    WebCore::CSSVariablesDeclaration * gself = (gdom_css_variables_declaration_get_private(self));
    switch (prop_id) {
     case PROP_CSS_TEXT:
    {
         gself->setCssText(WebCore::String::fromUTF8(g_value_get_string (value)) );
         break;
    }
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_css_variables_declaration__get_property_fn(GObject * object, guint prop_id,
                     GValue * value,
                     GParamSpec * pspec)
{
    GdomCSSVariablesDeclaration *self = GDOM_CSS_VARIABLES_DECLARATION(object);
    WebCore::CSSVariablesDeclaration * gself = (gdom_css_variables_declaration_get_private(self));
    switch (prop_id) {
    case PROP_CSS_TEXT:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->cssText()));
         break;
    }
    case PROP_LENGTH:
    {
         g_value_set_ulong (value, gself->length());
         break;
    }
    case PROP_PARENT_RULE:
    {
         PassRefPtr<WebCore::CSSRule> ptr = gself->parentRule();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_css_variables_declaration_class_init(GdomCSSVariablesDeclarationClass* requestClass)
{
    GObjectClass *g_klass = G_OBJECT_CLASS (requestClass);
    g_klass->finalize = gdom_css_variables_declaration_finalize;
    g_klass->set_property = gdom_css_variables_declaration__set_property_fn;
    g_klass->get_property = gdom_css_variables_declaration__get_property_fn;

    g_object_class_install_property (g_klass, PROP_CSS_TEXT,
            g_param_spec_string ("css_text", /* name */
                "css_variables_declaration_css_text", /* short description */
                "read-write  gchar * CSSVariablesDeclaration.css_text", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_LENGTH,
            g_param_spec_ulong ("length", /* name */
                "css_variables_declaration_length", /* short description */
                "read-only  gulong CSSVariablesDeclaration.length", /* longer - could do with some extra doc stuff here */
                 0,        /* min */
                G_MAXULONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_PARENT_RULE,
            g_param_spec_object ("parent_rule", /* name */
                "css_variables_declaration_parent_rule", /* short description */
                "read-only  GdomCSSRule * CSSVariablesDeclaration.parent_rule", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_CSS_RULE, /* gobject type */
                 (GParamFlags)G_PARAM_READABLE));





    /*g_type_class_remove_private(requestClass, sizeof(GdomCSSVariablesDeclarationPrivate));*/
}

static void gdom_css_variables_declaration_init(GdomCSSVariablesDeclaration* request)
{
    /*GdomCSSVariablesDeclarationPrivate* priv = GDOM_CSS_VARIABLES_DECLARATION_GET_PRIVATE(request);
    request->priv = priv;*/
}

} /* extern "C" */

